@@ -61,7 +61,7 @@ struct plplay {
     bool exit_thread;
 
     // settings / ui state
-    const struct pl_filter_preset *upscaler, *downscaler, *frame_mixer;
+    const struct pl_filter_preset *upscaler, *downscaler, *plane_scaler, *frame_mixer;
     struct pl_render_params params;
     struct pl_deband_params deband_params;
     struct pl_sigmoid_params sigmoid_params;
@@ -69,6 +69,7 @@ struct plplay {
     struct pl_peak_detect_params peak_detect_params;
     struct pl_color_map_params color_map_params;
     struct pl_dither_params dither_params;
+    struct pl_icc_params icc_params;
     struct pl_cone_params cone_params;
     struct pl_color_space target_color;
     struct pl_color_repr target_repr;
@@ -226,7 +227,9 @@ static bool init_codec(struct plplay *p)
     p->codec->get_buffer2 = pl_get_buffer2;
     p->codec->opaque = &p->win->gpu;
 #if LIBAVCODEC_VERSION_MAJOR < 60
-    p->codec->thread_safe_callbacks = 1;
+    AV_NOWARN_DEPRECATED({
+        p->codec->thread_safe_callbacks = 1;
+    });
 #endif
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(58, 113, 100)
     p->codec->export_side_data |= AV_CODEC_EXPORT_DATA_FILM_GRAIN;
@@ -569,6 +572,7 @@ int main(int argc, char **argv)
         .peak_detect_params = pl_peak_detect_default_params,
         .color_map_params = pl_color_map_default_params,
         .dither_params = pl_dither_default_params,
+        .icc_params = pl_icc_default_params,
         .cone_params = pl_vision_normal,
         .target_override = true,
     };
@@ -584,6 +588,7 @@ int main(int argc, char **argv)
     state.params.color_adjustment = &state.color_adjustment;
     state.params.color_map_params = &state.color_map_params;
     state.params.cone_params = &state.cone_params;
+    state.params.icc_params = &state.icc_params;
 
     // Enable dynamic parameters by default, due to plplay's heavy reliance on
     // GUI controls for dynamically adjusting render parameters.
@@ -645,6 +650,8 @@ int main(int argc, char **argv)
             p->upscaler = f;
         if (p->params.downscaler == f->filter)
             p->downscaler = f;
+        if (p->params.plane_upscaler == f->filter)
+            p->plane_scaler = f;
     }
 
     for (f = pl_frame_mixers; f->name; f++) {
@@ -722,6 +729,11 @@ error:
     pl_mpv_user_shader_destroy(&hook);
 }
 
+static const char *pscale_desc(const struct pl_filter_preset *f)
+{
+    return f->filter ? f->description : "None (Use regular upscaler)";
+}
+
 static void update_settings(struct plplay *p)
 {
     struct nk_context *nk = ui_get_context(p->ui);
@@ -824,6 +836,19 @@ static void update_settings(struct plplay *p)
                 nk_combo_end(nk);
             }
 
+            nk_label(nk, "Plane scaler:", NK_TEXT_LEFT);
+            if (nk_combo_begin_label(nk, pscale_desc(p->plane_scaler), nk_vec2(nk_widget_width(nk), 500))) {
+                nk_layout_row_dynamic(nk, 16, 1);
+                for (f = pl_scale_filters; f->name; f++) {
+                    if (!f->description)
+                        continue;
+                    if (nk_combo_item_label(nk, pscale_desc(f), NK_TEXT_LEFT))
+                        p->plane_scaler = f;
+                }
+                par->plane_upscaler = p->plane_scaler->filter;
+                nk_combo_end(nk);
+            }
+
             nk_label(nk, "Frame mixing:", NK_TEXT_LEFT);
             if (nk_combo_begin_label(nk, p->frame_mixer->description, nk_vec2(nk_widget_width(nk), 300))) {
                 nk_layout_row_dynamic(nk, 16, 1);
@@ -1086,6 +1111,7 @@ static void update_settings(struct plplay *p)
         if (nk_tree_push(nk, NK_TREE_NODE, "Output color space", NK_MINIMIZED)) {
             struct pl_color_space *tcol = &p->target_color;
             struct pl_color_repr *trepr = &p->target_repr;
+            struct pl_icc_params *iccpar = &p->icc_params;
             nk_layout_row_dynamic(nk, 24, 2);
             nk_checkbox_label(nk, "Enable", &p->target_override);
             bool reset = nk_button_label(nk, "Reset settings");
@@ -1159,6 +1185,7 @@ static void update_settings(struct plplay *p)
                 fix.hdr.min_luma /= 1000;
                 pl_color_space_infer(&fix);
                 tcol->hdr = fix.hdr;
+                iccpar->max_luma = fix.hdr.max_luma;
             } else {
                 reset_levels = true;
             }
@@ -1253,8 +1280,10 @@ static void update_settings(struct plplay *p)
                 };
             }
 
-            if (reset_levels)
+            if (reset_levels) {
                 tcol->hdr = (struct pl_hdr_metadata) {0};
+                *iccpar = pl_icc_default_params;
+            }
 
             nk_tree_pop(nk);
         }
@@ -1333,9 +1362,9 @@ static void update_settings(struct plplay *p)
             nk_checkbox_label(nk, "Preserve mixing cache", &par->preserve_mixing_cache);
             nk_checkbox_label(nk, "Disable linear scaling", &par->disable_linear_scaling);
             nk_checkbox_label(nk, "Disable built-in scalers", &par->disable_builtin_scalers);
-            nk_checkbox_label(nk, "Force-enable 3DLUT", &par->force_icc_lut);
             nk_checkbox_label(nk, "Force-enable dither", &par->force_dither);
             nk_checkbox_label(nk, "Disable FBOs / advanced rendering", &par->disable_fbos);
+            nk_checkbox_label(nk, "Force low-bit depth FBOs", &par->force_low_bit_depth_fbos);
             nk_checkbox_label(nk, "Disable constant hard-coding", &par->dynamic_constants);
             nk_checkbox_label(nk, "Ignore ICC profiles", &par->ignore_icc_profiles);
 
