@@ -433,6 +433,9 @@ void pl_color_space_infer(struct pl_color_space *space)
         space->sig_scale = 0;
     }
 
+    if (!pl_primaries_valid(&space->hdr.prim))
+        memset(&space->hdr.prim, 0, sizeof(space->hdr.prim));
+
     // Default the signal color space based on the nominal raw primaries
     pl_raw_primaries_merge(&space->hdr.prim, pl_raw_primaries_get(space->primaries));
 }
@@ -533,6 +536,21 @@ bool pl_raw_primaries_equal(const struct pl_raw_primaries *a,
            pl_cie_xy_equal(&a->white, &b->white);
 }
 
+bool pl_raw_primaries_similar(const struct pl_raw_primaries *a,
+                              const struct pl_raw_primaries *b)
+{
+    float delta = fabsf(a->red.x   - b->red.x)   +
+                  fabsf(a->red.y   - b->red.y)   +
+                  fabsf(a->green.x - b->green.x) +
+                  fabsf(a->green.y - b->green.y) +
+                  fabsf(a->blue.x  - b->blue.x)  +
+                  fabsf(a->blue.y  - b->blue.y)  +
+                  fabsf(a->white.x - b->white.x) +
+                  fabsf(a->white.y - b->white.y);
+
+    return delta < 0.001;
+}
+
 void pl_raw_primaries_merge(struct pl_raw_primaries *orig,
                             const struct pl_raw_primaries *update)
 {
@@ -988,6 +1006,15 @@ bool pl_primaries_superset(const struct pl_raw_primaries *a,
            test_point_gamut(b->blue, a);
 }
 
+bool pl_primaries_valid(const struct pl_raw_primaries *prim)
+{
+    // Test to see if the primaries form a valid triangle (nonzero area)
+    float area = (prim->blue.x - prim->green.x) * (prim->red.y  - prim->green.y)
+               - (prim->red.x  - prim->green.x) * (prim->blue.y - prim->green.y);
+
+    return fabs(area) > 1e-6 && test_point_gamut(prim->white, prim);
+}
+
 /* Fill in the Y, U, V vectors of a yuv-to-rgb conversion matrix
  * based on the given luma weights of the R, G and B components (lr, lg, lb).
  * lr+lg+lb is assumed to equal 1.
