@@ -71,6 +71,7 @@
 #include <libplacebo/shaders/colorspace.h>
 #include <libplacebo/shaders/custom.h>
 #include <libplacebo/shaders/film_grain.h>
+#include <libplacebo/shaders/icc.h>
 #include <libplacebo/shaders/lut.h>
 #include <libplacebo/shaders/sampling.h>
 #include <libplacebo/swapchain.h>
@@ -78,20 +79,14 @@
 #include <libplacebo/utils/frame_queue.h>
 #include <libplacebo/utils/upload.h>
 
-#ifdef PL_HAVE_LCMS
-#include "include/libplacebo/shaders/icc.h"
-#endif
-
 #ifdef PL_HAVE_VULKAN
-#include "include/libplacebo/vulkan.h"
+#include <libplacebo/vulkan.h>
 #endif
-
 #ifdef PL_HAVE_OPENGL
-#include "include/libplacebo/opengl.h"
+#include <libplacebo/opengl.h>
 #endif
-
 #ifdef PL_HAVE_D3D11
-#include "include/libplacebo/d3d11.h"
+#include <libplacebo/d3d11.h>
 #endif
 
 #pragma GCC visibility pop
@@ -181,6 +176,13 @@ static inline size_t pl_lcm(size_t x, size_t y)
         }                                                                       \
     } while (0)
 
+// Conditional abort() macro that depends on the configuration option
+#ifdef PL_DEBUG_ABORT
+# define pl_debug_abort() abort()
+#else
+# define pl_debug_abort()
+#endif
+
 // Refcounting helpers
 typedef _Atomic uint32_t pl_rc_t;
 #define pl_rc_init(rc)  atomic_init(rc, 1)
