@@ -230,6 +230,7 @@ static double box(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_box = {
     .weight    = box,
+    .name      = "box",
     .radius    = 1.0,
 };
 
@@ -241,6 +242,7 @@ static double triangle(const struct pl_filter_function *f, double x)
 const struct pl_filter_function pl_filter_function_triangle = {
     .resizable = true,
     .weight    = triangle,
+    .name      = "triangle",
     .radius    = 1.0,
 };
 
@@ -251,6 +253,7 @@ static double cosine(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_cosine = {
     .weight = cosine,
+    .name   = "cosine",
     .radius = M_PI / 2.0,
 };
 
@@ -261,6 +264,7 @@ static double hann(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_hann = {
     .weight = hann,
+    .name   = "hann",
     .radius = 1.0,
 };
 
@@ -271,6 +275,7 @@ static double hamming(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_hamming = {
     .weight = hamming,
+    .name   = "hamming",
     .radius = 1.0,
 };
 
@@ -281,6 +286,7 @@ static double welch(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_welch = {
     .weight = welch,
+    .name   = "welch",
     .radius = 1.0,
 };
 
@@ -307,6 +313,7 @@ static double kaiser(const struct pl_filter_function *f, double x)
 const struct pl_filter_function pl_filter_function_kaiser = {
     .tunable = {true},
     .weight  = kaiser,
+    .name    = "kaiser",
     .radius  = 1.0,
     .params  = {2.0},
 };
@@ -322,6 +329,7 @@ static double blackman(const struct pl_filter_function *f, double x)
 const struct pl_filter_function pl_filter_function_blackman = {
     .tunable = {true},
     .weight  = blackman,
+    .name    = "blackman",
     .radius  = 1.0,
     .params  = {0.16},
 };
@@ -334,6 +342,7 @@ static double bohman(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_bohman = {
     .weight = bohman,
+    .name   = "bohman",
     .radius = 1.0,
 };
 
@@ -346,6 +355,7 @@ const struct pl_filter_function pl_filter_function_gaussian = {
     .resizable = true,
     .tunable   = {true},
     .weight    = gaussian,
+    .name      = "gaussian",
     .radius    = 2.0,
     .params    = {1.0},
 };
@@ -361,6 +371,7 @@ static double quadratic(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_quadratic = {
     .weight = quadratic,
+    .name   = "quadratic",
     .radius = 1.5,
 };
 
@@ -375,6 +386,7 @@ static double sinc(const struct pl_filter_function *f, double x)
 const struct pl_filter_function pl_filter_function_sinc = {
     .resizable = true,
     .weight    = sinc,
+    .name      = "sinc",
     .radius    = 1.0,
 };
 
@@ -389,6 +401,7 @@ static double jinc(const struct pl_filter_function *f, double x)
 const struct pl_filter_function pl_filter_function_jinc = {
     .resizable = true,
     .weight    = jinc,
+    .name      = "jinc",
     .radius    = 1.2196698912665045, // first zero
 };
 
@@ -403,6 +416,7 @@ static double sphinx(const struct pl_filter_function *f, double x)
 const struct pl_filter_function pl_filter_function_sphinx = {
     .resizable = true,
     .weight    = sphinx,
+    .name      = "sphinx",
     .radius    = 1.4302966531242027, // first zero
 };
 
@@ -431,6 +445,7 @@ static double bcspline(const struct pl_filter_function *f, double x)
 const struct pl_filter_function pl_filter_function_bcspline = {
     .tunable = {true, true},
     .weight  = bcspline,
+    .name    = "bcspline",
     .radius  = 2.0,
     .params  = {0.5, 0.5},
 };
@@ -438,6 +453,7 @@ const struct pl_filter_function pl_filter_function_bcspline = {
 const struct pl_filter_function pl_filter_function_catmull_rom = {
     .tunable = {true, true},
     .weight  = bcspline,
+    .name    = "catmull_rom",
     .radius  = 2.0,
     .params  = {0.0, 0.5},
 };
@@ -445,6 +461,7 @@ const struct pl_filter_function pl_filter_function_catmull_rom = {
 const struct pl_filter_function pl_filter_function_mitchell = {
     .tunable = {true, true},
     .weight  = bcspline,
+    .name    = "mitchell",
     .radius  = 2.0,
     .params  = {1/3.0, 1/3.0},
 };
@@ -452,6 +469,7 @@ const struct pl_filter_function pl_filter_function_mitchell = {
 const struct pl_filter_function pl_filter_function_robidoux = {
     .tunable = {true, true},
     .weight  = bcspline,
+    .name    = "robidoux",
     .radius  = 2.0,
     .params  = {12 / (19 + 9 * M_SQRT2), 113 / (58 + 216 * M_SQRT2)},
 };
@@ -459,6 +477,7 @@ const struct pl_filter_function pl_filter_function_robidoux = {
 const struct pl_filter_function pl_filter_function_robidouxsharp = {
     .tunable = {true, true},
     .weight  = bcspline,
+    .name    = "robidouxsharp",
     .radius  = 2.0,
     .params  = {6 / (13 + 7 * M_SQRT2), 7 / (2 + 12 * M_SQRT2)},
 };
@@ -474,6 +493,7 @@ static double bicubic(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_bicubic = {
     .weight = bicubic,
+    .name   = "bicubic",
     .radius = 2.0,
 };
 
@@ -488,6 +508,7 @@ static double spline16(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_spline16 = {
     .weight = spline16,
+    .name   = "spline16",
     .radius = 2.0,
 };
 
@@ -504,6 +525,7 @@ static double spline36(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_spline36 = {
     .weight = spline36,
+    .name   = "spline36",
     .radius = 3.0,
 };
 
@@ -522,6 +544,7 @@ static double spline64(const struct pl_filter_function *f, double x)
 
 const struct pl_filter_function pl_filter_function_spline64 = {
     .weight = spline64,
+    .name   = "spline64",
     .radius = 4.0,
 };
 
@@ -563,77 +586,92 @@ const int pl_num_filter_function_presets = PL_ARRAY_SIZE(pl_filter_function_pres
 // Built-in filter function presets
 const struct pl_filter_config pl_filter_spline16 = {
     .kernel = &pl_filter_function_spline16,
+    .name   = "spline16",
 };
 
 const struct pl_filter_config pl_filter_spline36 = {
     .kernel = &pl_filter_function_spline36,
+    .name   = "spline36",
 };
 
 const struct pl_filter_config pl_filter_spline64 = {
     .kernel = &pl_filter_function_spline64,
+    .name   = "spline64"
 };
 
 const struct pl_filter_config pl_filter_nearest = {
     .kernel = &pl_filter_function_box,
+    .name   = "nearest",
 };
 
 const struct pl_filter_config pl_filter_bilinear = {
     .kernel = &pl_filter_function_triangle,
+    .name   = "bilinear",
 };
 
 const struct pl_filter_config pl_filter_gaussian = {
     .kernel = &pl_filter_function_gaussian,
+    .name   = "gaussian",
 };
 
 // Sinc configured to three taps
 static const struct pl_filter_function sinc3 = {
     .resizable = true,
     .weight    = sinc,
+    .name      = "sinc3",
     .radius    = 3.0,
 };
 
 const struct pl_filter_config pl_filter_sinc = {
     .kernel = &sinc3,
+    .name   = "sinc",
 };
 
 const struct pl_filter_config pl_filter_lanczos = {
     .kernel = &sinc3,
     .window = &pl_filter_function_sinc,
+    .name   = "lanczos",
 };
 
 const struct pl_filter_config pl_filter_ginseng = {
     .kernel = &sinc3,
     .window = &pl_filter_function_jinc,
+    .name   = "ginseng",
 };
 
 // Jinc configured to three taps
 static const struct pl_filter_function jinc3 = {
     .resizable = true,
     .weight    = jinc,
+    .name      = "jinc3",
     .radius    = 3.2383154841662362, // third zero
 };
 
 const struct pl_filter_config pl_filter_ewa_jinc = {
     .kernel = &jinc3,
-    .polar = true,
+    .polar  = true,
+    .name   = "ewa_jinc",
 };
 
 const struct pl_filter_config pl_filter_ewa_lanczos = {
     .kernel = &jinc3,
     .window = &pl_filter_function_jinc,
-    .polar = true,
+    .polar  = true,
+    .name   = "ewa_lanczos",
 };
 
 const struct pl_filter_config pl_filter_ewa_ginseng = {
     .kernel = &jinc3,
     .window = &pl_filter_function_sinc,
-    .polar = true,
+    .polar  = true,
+    .name   = "ewa_ginseng",
 };
 
 const struct pl_filter_config pl_filter_ewa_hann = {
     .kernel = &jinc3,
     .window = &pl_filter_function_hann,
-    .polar = true,
+    .polar  = true,
+    .name   = "ewa_hann",
 };
 
 const struct pl_filter_config pl_filter_haasnsoft = {
@@ -642,44 +680,53 @@ const struct pl_filter_config pl_filter_haasnsoft = {
     // The blur is tuned to equal out orthogonal and diagonal contributions
     // on a regular grid. This has the effect of almost completely killing
     // aliasing.
-    .blur = 1.11,
-    .polar = true,
+    .blur   = 1.11,
+    .polar  = true,
+    .name   = "haasnsoft",
 };
 
 // Spline family
 const struct pl_filter_config pl_filter_bicubic = {
     .kernel = &pl_filter_function_bicubic,
+    .name   = "bicubic",
 };
 
 const struct pl_filter_config pl_filter_catmull_rom = {
     .kernel = &pl_filter_function_catmull_rom,
+    .name   = "catmull_rom",
 };
 
 const struct pl_filter_config pl_filter_mitchell = {
     .kernel = &pl_filter_function_mitchell,
+    .name   = "mitchell",
 };
 
 const struct pl_filter_config pl_filter_mitchell_clamp = {
     .kernel = &pl_filter_function_mitchell,
-    .clamp = 1.0,
+    .clamp  = 1.0,
+    .name   = "mitchell_clamp",
 };
 
 const struct pl_filter_config pl_filter_robidoux = {
     .kernel = &pl_filter_function_robidoux,
+    .name   = "robidoux",
 };
 
 const struct pl_filter_config pl_filter_robidouxsharp = {
     .kernel = &pl_filter_function_robidouxsharp,
+    .name   = "robidouxsharp",
 };
 
 const struct pl_filter_config pl_filter_ewa_robidoux = {
     .kernel = &pl_filter_function_robidoux,
-    .polar = true,
+    .polar  = true,
+    .name   = "ewa_robidoux",
 };
 
 const struct pl_filter_config pl_filter_ewa_robidouxsharp = {
     .kernel = &pl_filter_function_robidouxsharp,
-    .polar = true,
+    .polar  = true,
+    .name   = "ewa_robidouxsharp",
 };
 
 // Named filter configs
