@@ -28,6 +28,7 @@
           PL_ERR(gpu, "Validation failed: %s (%s:%d)",          \
                   #expr, __FILE__, __LINE__);                   \
           pl_log_stack_trace(gpu->log, PL_LOG_ERR);             \
+          pl_debug_abort();                                     \
           goto error;                                           \
       }                                                         \
   } while (0)
@@ -541,7 +542,7 @@ next_fmt: ; // equivalent to `continue`
     }
 
     // ran out of formats
-    PL_DEBUG(gpu, "No matching format found");
+    PL_TRACE(gpu, "No matching format found");
     return NULL;
 }
 
@@ -902,6 +903,7 @@ static bool fix_tex_transfer(pl_gpu gpu, struct pl_tex_transfer_params *params)
     if (params->buf) {
         pl_buf buf = params->buf;
         size_t size = pl_tex_transfer_size(params);
+        require(params->buf_offset + size >= params->buf_offset); // overflow check
         require(params->buf_offset + size <= buf->params.size);
         require(gpu->limits.buf_transfer);
     }
@@ -1379,6 +1381,35 @@ const struct pl_blend_params pl_alpha_overlay = {
     .dst_alpha  = PL_BLEND_ONE_MINUS_SRC_ALPHA,
 };
 
+static inline void log_shader_sources(pl_log log, enum pl_log_level level,
+                                      const struct pl_pass_params *params)
+{
+    if (!pl_msg_test(log, level) || !params->glsl_shader)
+        return;
+
+    switch (params->type) {
+    case PL_PASS_RASTER:
+        if (!params->vertex_shader)
+            return;
+        pl_msg(log, level, "vertex shader source:");
+        pl_msg_source(log, level, params->vertex_shader);
+        pl_msg(log, level, "fragment shader source:");
+        pl_msg_source(log, level, params->glsl_shader);
+        return;
+
+    case PL_PASS_COMPUTE:
+        pl_msg(log, level, "compute shader source:");
+        pl_msg_source(log, level, params->glsl_shader);
+        return;
+
+    case PL_PASS_INVALID:
+    case PL_PASS_TYPE_COUNT:
+        break;
+    }
+
+    pl_unreachable();
+}
+
 pl_pass pl_pass_create(pl_gpu gpu, const struct pl_pass_params *params)
 {
     struct pl_pass_params fixed;
@@ -1446,9 +1477,15 @@ pl_pass pl_pass_create(pl_gpu gpu, const struct pl_pass_params *params)
     require(params->push_constants_size == PL_ALIGN2(params->push_constants_size, 4));
 
     const struct pl_gpu_fns *impl = PL_PRIV(gpu);
-    return impl->pass_create(gpu, params);
+    log_shader_sources(gpu->log, PL_LOG_DEBUG, params);
+    pl_pass pass = impl->pass_create(gpu, params);
+    if (!pass)
+        goto error;
+
+    return pass;
 
 error:
+    log_shader_sources(gpu->log, PL_LOG_ERR, params);
     return NULL;
 }
 
@@ -1838,7 +1875,7 @@ bool pl_tex_upload_texel(pl_gpu gpu, pl_dispatch dp,
 
     // fmt->texel_align contains the size of an individual color value
     assert(fmt->texel_size == fmt->num_components * fmt->texel_align);
-    GLSL("vec4 color = vec4(0.0);                                       \n"
+    GLSL("vec4 color = vec4(0.0, 0.0, 0.0, 1.0);                        \n"
          "ivec3 pos = ivec3(gl_GlobalInvocationID) + ivec3(%d, %d, %d); \n"
          "int base = pos.z * %s + pos.y * %s + pos.x * %s;              \n",
          params->rc.x0, params->rc.y0, params->rc.z0,
