@@ -140,6 +140,9 @@ extern const struct pl_matrix2x2 pl_matrix2x2_identity;
 void pl_matrix2x2_apply(const struct pl_matrix2x2 *mat, float vec[2]);
 void pl_matrix2x2_apply_rc(const struct pl_matrix2x2 *mat, struct pl_rect2df *rc);
 
+void pl_matrix2x2_mul(struct pl_matrix2x2 *a, const struct pl_matrix2x2 *b);
+void pl_matrix2x2_rmul(const struct pl_matrix2x2 *a, struct pl_matrix2x2 *b);
+
 struct pl_transform2x2 {
     struct pl_matrix2x2 mat;
     float c[2];
@@ -150,6 +153,9 @@ extern const struct pl_transform2x2 pl_transform2x2_identity;
 void pl_transform2x2_apply(const struct pl_transform2x2 *t, float vec[2]);
 void pl_transform2x2_apply_rc(const struct pl_transform2x2 *t, struct pl_rect2df *rc);
 
+void pl_transform2x2_mul(struct pl_transform2x2 *a, const struct pl_transform2x2 *b);
+void pl_transform2x2_rmul(const struct pl_transform2x2 *a, struct pl_transform2x2 *b);
+
 // Helper functions for dealing with aspect ratios and stretched/scaled rects.
 
 // Return the (absolute) aspect ratio (width/height) of a given pl_rect2df.
@@ -208,9 +214,7 @@ static inline pl_rotation pl_rotation_normalize(pl_rotation rot)
 // For example, calling this with PL_ROTATION_90 will correspond to rotating
 // the coordinate system 90° to the right (so the x axis becomes the y axis).
 //
-// The resulting rect is re-normalized in the same coordinate system, so this
-// is not the same as simply applying the rotation matrix generated by
-// `pl_rotation_matrix` to the rect.
+// The resulting rect is re-normalized in the same coordinate system.
 void pl_rect2df_rotate(struct pl_rect2df *rc, pl_rotation rot);
 
 // Returns the aspect ratio in a rotated frame of reference.
