@@ -105,7 +105,8 @@ struct pl_vulkan_queue {
 };
 
 // Structure representing the actual vulkan device and associated GPU instance
-typedef const PL_STRUCT(pl_vulkan) {
+typedef const PL_STRUCT(pl_vulkan) *pl_vulkan;
+PL_STRUCT(pl_vulkan) {
     pl_gpu gpu;
 
     // The vulkan objects in use. The user may use this for their own purposes,
@@ -128,14 +129,7 @@ typedef const PL_STRUCT(pl_vulkan) {
     // The device features that were enabled at device creation time.
     const VkPhysicalDeviceFeatures2 *features;
 
-    // The explicit queue families we are using to provide a given capability,
-    // or {0} if no appropriate dedicated queue family exists for this
-    // operation type.
-    //
-    // It's guaranteed that `queue_graphics` is always set, but the existence
-    // of the other two is optional, and libplacebo will only set them if
-    // they are different from the graphics queue. Note that queue_compute
-    // and queue_transfer may refer to the same queue family index.
+    // The explicit queue families we are using to provide a given capability.
     struct pl_vulkan_queue queue_graphics; // provides VK_QUEUE_GRAPHICS_BIT
     struct pl_vulkan_queue queue_compute;  // provides VK_QUEUE_COMPUTE_BIT
     struct pl_vulkan_queue queue_transfer; // provides VK_QUEUE_TRANSFER_BIT
@@ -144,7 +138,14 @@ typedef const PL_STRUCT(pl_vulkan) {
     // queue counts in list form. This list does not contain duplicates.
     const struct pl_vulkan_queue *queues;
     int num_queues;
-} *pl_vulkan;
+
+    // Functions for locking a queue. These must be used to lock VkQueues for
+    // submission or other related operations when sharing the VkDevice between
+    // multiple threads, Using this on queue families or indices not contained
+    // in `queues` is undefined behavior.
+    void (*lock_queue)(pl_vulkan vk, int qf, int qidx);
+    void (*unlock_queue)(pl_vulkan vk, int qf, int qidx);
+};
 
 struct pl_vulkan_params {
     // The vulkan instance. Optional, if NULL then libplacebo will internally
@@ -401,6 +402,12 @@ struct pl_vulkan_import_params {
     // *must* be created with the `timelineSemaphore` feature enabled.
     const VkPhysicalDeviceFeatures2 *features;
 
+    // Functions for locking a queue. If set, these will be used instead of
+    // libplacebo's internal functions for `pl_vulkan.(un)lock_queue`.
+    void (*lock_queue)(void *ctx, int qf, int qidx);
+    void (*unlock_queue)(void *ctx, int qf, int qidx);
+    void *queue_ctx;
+
     // --- Misc/debugging options
 
     // Restrict specific features to e.g. work around driver bugs, or simply
