@@ -18,6 +18,26 @@
 #pragma once
 
 #include "../common.h"
-#include "../context.h"
+#include "../log.h"
+#include "../gpu.h"
 
 #include <epoxy/gl.h>
+
+#ifdef EPOXY_HAS_EGL
+#include <epoxy/egl.h>
+#endif
+
+struct gl_cb {
+    void (*callback)(void *priv);
+    void *priv;
+    GLsync sync;
+};
+
+struct fbo_format {
+    pl_fmt fmt;
+    const struct gl_format *glfmt;
+};
+
+// For locking/unlocking
+bool gl_make_current(pl_opengl gl);
+void gl_release_current(pl_opengl gl);
