@@ -252,7 +252,7 @@ pl_gpu pl_gpu_create_gl(pl_log log, pl_opengl gl, const struct pl_opengl_params
     limits->align_tex_xfer_offset = 32;
     limits->align_tex_xfer_pitch = 1;
     limits->fragment_queues = 1;
-    limits->compute_queues = 1;
+    limits->compute_queues = glsl->compute ? 1 : 0;
     if (gl_test_ext(gpu, "GL_EXT_unpack_subimage", 11, 30))
         limits->align_tex_xfer_pitch = 4;
 
