@@ -151,6 +151,9 @@ void sh_append_str(pl_shader sh, enum pl_shader_buf buf, pl_str str);
 #define GLSLF(...) sh_append(sh, SH_BUF_FOOTER, __VA_ARGS__)
 
 // Attach a description to a shader
+void sh_describef(pl_shader sh, const char *fmt, ...)
+    PL_PRINTF(2, 3);
+
 static inline void sh_describe(pl_shader sh, const char *desc)
 {
     PL_ARRAY_APPEND(sh, sh->steps, desc);
@@ -168,7 +171,6 @@ enum pl_shader_obj_type {
     PL_SHADER_OBJ_TONE_MAP,
     PL_SHADER_OBJ_SAMPLER,
     PL_SHADER_OBJ_DITHER,
-    PL_SHADER_OBJ_ICC,
     PL_SHADER_OBJ_LUT,
     PL_SHADER_OBJ_AV1_GRAIN,
     PL_SHADER_OBJ_FILM_GRAIN,
@@ -213,6 +215,10 @@ struct sh_lut_params {
     enum sh_lut_method method;
     enum pl_var_type type;
 
+    // For SH_LUT_TEXTURE, this can be used to override the texture's internal
+    // format, in which case it takes precedence over the default for `type`.
+    pl_fmt fmt;
+
     // LUT dimensions. Unused dimensions may be left as 0.
     int width;
     int height;
@@ -239,7 +245,7 @@ struct sh_lut_params {
     // be computed, which happens whenever the size is changed, the shader
     // object is invalidated, or `update` is set to true.
     //
-    // Note: Interpretation of `data` is according to `pl_var_type`.
+    // Note: Interpretation of `data` is according to `type` and `fmt`.
     void (*fill)(void *data, const struct sh_lut_params *params);
     void *priv;
 };
