@@ -113,9 +113,11 @@ enum szexp_op {
     SZEXP_OP_SUB,
     SZEXP_OP_MUL,
     SZEXP_OP_DIV,
+    SZEXP_OP_MOD,
     SZEXP_OP_NOT,
     SZEXP_OP_GT,
     SZEXP_OP_LT,
+    SZEXP_OP_EQ,
 };
 
 enum szexp_tag {
@@ -191,9 +193,11 @@ static bool parse_rpn_szexpr(pl_str line, struct szexp out[MAX_SZEXP_SIZE])
         case '-': exp->tag = SZEXP_OP2; exp->val.op = SZEXP_OP_SUB; continue;
         case '*': exp->tag = SZEXP_OP2; exp->val.op = SZEXP_OP_MUL; continue;
         case '/': exp->tag = SZEXP_OP2; exp->val.op = SZEXP_OP_DIV; continue;
+        case '%': exp->tag = SZEXP_OP2; exp->val.op = SZEXP_OP_MOD; continue;
         case '!': exp->tag = SZEXP_OP1; exp->val.op = SZEXP_OP_NOT; continue;
         case '>': exp->tag = SZEXP_OP2; exp->val.op = SZEXP_OP_GT;  continue;
         case '<': exp->tag = SZEXP_OP2; exp->val.op = SZEXP_OP_LT;  continue;
+        case '=': exp->tag = SZEXP_OP2; exp->val.op = SZEXP_OP_EQ;  continue;
         }
 
         if (word.buf[0] >= '0' && word.buf[0] <= '9') {
@@ -259,8 +263,10 @@ static bool pl_eval_szexpr(pl_log log, void *priv,
             case SZEXP_OP_SUB: res = op1 - op2; break;
             case SZEXP_OP_MUL: res = op1 * op2; break;
             case SZEXP_OP_DIV: res = op1 / op2; break;
+            case SZEXP_OP_MOD: res = fmodf(op1, op2); break;
             case SZEXP_OP_GT:  res = op1 > op2; break;
             case SZEXP_OP_LT:  res = op1 < op2; break;
+            case SZEXP_OP_EQ:  res = fabsf(op1 - op2) <= 1e-6 * fmaxf(op1, op2); break;
             case SZEXP_OP_NOT: pl_unreachable();
             }
 
@@ -834,7 +840,6 @@ static pl_str pl_stage_to_mp(enum pl_hook_stage stage)
 
     case PL_HOOK_LINEAR:        return pl_str0("LINEAR");
     case PL_HOOK_SIGMOID:       return pl_str0("SIGMOID");
-    case PL_HOOK_PRE_OVERLAY:   return pl_str0("PREOVERLAY"); // Note: doesn't exist!
     case PL_HOOK_PRE_KERNEL:    return pl_str0("PREKERNEL");
     case PL_HOOK_POST_KERNEL:   return pl_str0("POSTKERNEL");
 
@@ -874,6 +879,7 @@ struct hook_priv {
     // Dynamic per pass
     enum pl_hook_stage save_stages;
     PL_ARRAY(struct pass_tex) pass_textures;
+    pl_shader trc_helper;
 
     // State for PRNG/frame count
     int frame_count;
@@ -1233,6 +1239,22 @@ static struct pl_hook_res hook_hook(void *priv, const struct pl_hook_params *par
             .data = tex_off,
         }));
 
+        // Helper sub-shaders
+        uint64_t sh_id = SH_PARAMS(sh).id;
+        pl_shader_reset(p->trc_helper, pl_shader_params(
+            .id = ++sh_id,
+            .gpu = p->gpu,
+        ));
+        pl_shader_linearize(p->trc_helper, params->orig_color);
+        GLSLH("#define linearize %s \n", sh_subpass(sh, p->trc_helper));
+
+        pl_shader_reset(p->trc_helper, pl_shader_params(
+            .id = ++sh_id,
+            .gpu = p->gpu,
+        ));
+        pl_shader_delinearize(p->trc_helper, params->orig_color);
+        GLSLH("#define delinearize %s \n", sh_subpass(sh, p->trc_helper));
+
         // Load and run the user shader itself
         sh_append_str(sh, SH_BUF_HEADER, hook->pass_body);
         sh_describe(sh, pl_strdup0(SH_TMP(sh), hook->pass_desc));
@@ -1353,6 +1375,7 @@ const struct pl_hook *pl_mpv_user_shader_parse(pl_gpu gpu,
         .log = gpu->log,
         .gpu = gpu,
         .alloc = hook,
+        .trc_helper = pl_shader_alloc(gpu->log, NULL),
         .prng_state = {
             // Determined by fair die roll
             0xb76d71f9443c228allu, 0x93a02092fc4807e8llu,
@@ -1481,6 +1504,7 @@ void pl_mpv_user_shader_destroy(const struct pl_hook **hookp)
         }
     }
 
+    pl_shader_free(&p->trc_helper);
     pl_free((void *) hook);
     *hookp = NULL;
 }
