@@ -382,22 +382,24 @@ bool pl_shader_sample_oversample(pl_shader sh, const struct pl_sample_src *src,
     GLSL("// pl_shader_sample_oversample                \n"
          "vec4 color;                                   \n"
          "{                                             \n"
-         "vec2 pt = %s;                                 \n"
-         "vec2 pos = %s - vec2(0.5) * pt;               \n"
-         "vec2 fcoord = fract(pos * %s - vec2(0.5));    \n"
-         "vec2 coeff = fcoord * %s;                     \n",
-         pt, pos, size, ratio);
+         "vec2 pt = %s, size = %s, pos = %s;            \n"
+         "vec2 fcoord = fract(pos * size - vec2(0.5));  \n"
+         "vec2 coeff = (fcoord - vec2(0.5)) * %s;       \n"
+         "coeff = clamp(coeff + vec2(0.5), 0.0, 1.0);   \n",
+         pt, size, pos, ratio);
 
-    if (threshold > 0.0) {
-        threshold = PL_MIN(threshold, 1.0);
+    if (threshold > 0) {
+        threshold = PL_MIN(threshold, 0.5f);
         ident_t thresh = sh_const_float(sh, "threshold", threshold);
-        GLSL("coeff = (coeff - %s) / (1.0 - 2.0 * %s);  \n",
+        GLSL("coeff = mix(coeff, vec2(0.0),             \n"
+             "    lessThan(coeff, vec2(%s)));           \n"
+             "coeff = mix(coeff, vec2(1.0),             \n"
+             "    greaterThan(coeff, vec2(1.0 - %s)));  \n",
              thresh, thresh);
     }
 
     // Compute the right output blend of colors
-    GLSL("coeff = clamp(coeff, 0.0, 1.0);               \n"
-         "pos += (coeff - fcoord) * pt;                 \n"
+    GLSL("pos += (coeff - fcoord) * pt;                 \n"
          "color = vec4(%s) * %s(%s, pos);               \n"
          "}                                             \n",
          SH_FLOAT(scale), fn, tex);
@@ -405,6 +407,32 @@ bool pl_shader_sample_oversample(pl_shader sh, const struct pl_sample_src *src,
     return true;
 }
 
+static void describe_filter(pl_shader sh, const struct pl_filter_config *cfg,
+                            const char *stage, float rx, float ry)
+{
+    const char *dir;
+    if (rx > 1 && ry > 1) {
+        dir = "up";
+    } else if (rx < 1 && ry < 1) {
+        dir = "down";
+    } else if (rx == 1 && ry == 1) {
+        dir = "noop";
+    } else {
+        dir = "ana";
+    }
+
+    if (cfg->name) {
+        sh_describef(sh, "%s %sscaling (%s)", stage, dir, cfg->name);
+    } else if (cfg->window) {
+        sh_describef(sh, "%s %sscaling (%s+%s)", stage, dir,
+                     PL_DEF(cfg->kernel->name, "unknown"),
+                     PL_DEF(cfg->window->name, "unknown"));
+    } else {
+        sh_describef(sh, "%s %sscaling (%s)", stage, dir,
+                     PL_DEF(cfg->kernel->name, "unknown"));
+    }
+}
+
 static bool filter_compat(pl_filter filter, float inv_scale,
                           int lut_entries, float cutoff,
                           const struct pl_filter_config *params)
@@ -502,30 +530,6 @@ bool pl_shader_sample_polar(pl_shader sh, const struct pl_sample_src *src,
         return false;
     }
 
-    bool has_compute = sh_glsl(sh).compute && !params->no_compute;
-    has_compute &= sh_glsl(sh).version >= 130; // needed for round()
-    if (!src->tex && has_compute) {
-        // FIXME: Could maybe solve this by communicating the wbase from
-        // invocation 0 to the rest of the workgroup using shmem, which would
-        // also allow us to avoid the use of the hacky %s_map below.
-        PL_WARN(sh, "Combining pl_shader_sample_polar with the sampler2D "
-                "interface prevents the use of compute shaders, which is a "
-                "potentially massive performance hit. If you're sure you want "
-                "this, set `params.no_compute` to suppress this warning.");
-        has_compute = false;
-    }
-
-    bool flipped = src->rect.x0 > src->rect.x1 || src->rect.y0 > src->rect.y1;
-    if (flipped && has_compute) {
-        // FIXME: I'm sure this case could actually be supported with some
-        // extra math in the positional calculations, should implement it
-        PL_WARN(sh, "Trying to use a flipped src.rect with polar sampling! "
-                "This prevents the use of compute shaders, which is a "
-                "potentially massive performance hit. If you're really sure you "
-                "want this, set `params.no_compute` to suppress this warning.");
-        has_compute = false;
-    }
-
     uint8_t comp_mask;
     float rx, ry, scale;
     ident_t src_tex, pos, size, pt;
@@ -567,7 +571,7 @@ bool pl_shader_sample_polar(pl_shader sh, const struct pl_sample_src *src,
         }
     }
 
-    sh_describe(sh, "polar scaling");
+    describe_filter(sh, &params->filter, "polar", rx, ry);
     GLSL("// pl_shader_sample_polar                     \n"
          "vec4 color = vec4(0.0);                       \n"
          "{                                             \n"
@@ -589,15 +593,20 @@ bool pl_shader_sample_polar(pl_shader sh, const struct pl_sample_src *src,
     // just use as many threads as possible.
     const int bw = 32, bh = sh_glsl(sh).max_group_threads / bw;
 
-    // We need to sample everything from base_min to base_max, so make sure
-    // we have enough room in shmem
-    int iw = (int) ceil(bw / rx) + padding + 1,
-        ih = (int) ceil(bh / ry) + padding + 1;
+    // We need to sample everything from base_min to base_max, so make sure we
+    // have enough room in shmem. The extra margin on the ceilf guards against
+    // floating point inaccuracy on near-integer scaling ratios.
+    const float margin = 1e-5;
+    int iw = (int) ceilf(bw / rx - margin) + padding + 1,
+        ih = (int) ceilf(bh / ry - margin) + padding + 1;
 
     ident_t in = NULL;
     int num_comps = __builtin_popcount(comp_mask);
-    int shmem_req = iw * ih * num_comps * sizeof(float);
-    bool is_compute = has_compute && sh_try_compute(sh, bw, bh, false, shmem_req);
+    int shmem_req = (iw * ih * num_comps + 2) * sizeof(float);
+    bool is_compute = !params->no_compute &&
+                      sh_glsl(sh).compute &&
+                      sh_glsl(sh).version >= 130 && // needed for round()
+                      sh_try_compute(sh, bw, bh, false, shmem_req);
 
     // For compute shaders, which read the input texels primarily from shmem,
     // using a texture-based LUT is better. For the fragment shader fallback
@@ -624,11 +633,21 @@ bool pl_shader_sample_polar(pl_shader sh, const struct pl_sample_src *src,
     ident_t radius_c = sh_const_float(sh, "radius", obj->filter->radius);
 
     if (is_compute) {
+
         // Compute shader kernel
-        GLSL("vec2 wpos = %s_map(gl_WorkGroupID * gl_WorkGroupSize);        \n"
-             "vec2 wbase = wpos - pt * fract(wpos * size - vec2(0.5));      \n"
-             "ivec2 rel = ivec2(round((base - wbase) * size));              \n",
-             pos);
+        GLSL("uvec2 base_id = uvec2(0u); \n");
+        if (src->rect.x0 > src->rect.x1)
+            GLSL("base_id.x = gl_WorkGroupSize.x - 1u; \n");
+        if (src->rect.y0 > src->rect.y1)
+            GLSL("base_id.y = gl_WorkGroupSize.y - 1u; \n");
+
+        in = sh_fresh(sh, "in");
+        GLSLH("shared vec2 %s_base; \n", in);
+        GLSL("if (gl_LocalInvocationID.xy == base_id)               \n"
+             "    %s_base = base;                                   \n"
+             "barrier();                                            \n"
+             "ivec2 rel = ivec2(round((base - %s_base) * size));    \n",
+             in, in);
 
         ident_t iw_c = sh_const(sh, (struct pl_shader_const) {
             .type = PL_VAR_SINT,
@@ -647,10 +666,9 @@ bool pl_shader_sample_polar(pl_shader sh, const struct pl_sample_src *src,
         // Load all relevant texels into shmem
         GLSL("for (int y = int(gl_LocalInvocationID.y); y < %s; y += %d) {  \n"
              "for (int x = int(gl_LocalInvocationID.x); x < %s; x += %d) {  \n"
-             "c = %s(%s, wbase + pt * vec2(x - %d, y - %d));                \n",
-             ih_c, bh, iw_c, bw, fn, src_tex, offset, offset);
+             "c = %s(%s, %s_base + pt * vec2(x - %d, y - %d));              \n",
+             ih_c, bh, iw_c, bw, fn, src_tex, in, offset, offset);
 
-        in = sh_fresh(sh, "in");
         for (uint8_t comps = comp_mask; comps;) {
             uint8_t c = __builtin_ctz(comps);
             GLSLH("shared float %s%d[%s * %s]; \n", in, c, ih_c, iw_c);
@@ -900,11 +918,11 @@ bool pl_shader_sample_ortho(pl_shader sh, int pass,
     };
 
     static const char *names[PL_SEP_PASSES] = {
-        [PL_SEP_HORIZ] = "ortho scaling (horiz)",
-        [PL_SEP_VERT]  = "ortho scaling (vert)",
+        [PL_SEP_HORIZ] = "ortho (horiz)",
+        [PL_SEP_VERT]  = "ortho (vert)",
     };
 
-    sh_describe(sh, names[pass]);
+    describe_filter(sh, &params->filter, names[pass], ratio[pass], ratio[pass]);
     GLSL("// pl_shader_sample_ortho                        \n"
          "vec4 color = vec4(0.0);                          \n"
          "{                                                \n"
