@@ -580,28 +580,6 @@ static void pl_shader_tests(pl_gpu gpu)
     pl_dispatch_abort(dp, &sh);
     pl_shader_obj_destroy(&peak_state);
 
-#ifdef PL_HAVE_LCMS
-    // Test the use of ICC profiles if available
-    sh = pl_dispatch_begin(dp);
-    pl_shader_sample_nearest(sh, pl_sample_src( .tex = src ));
-
-    pl_shader_obj icc = NULL;
-    struct pl_icc_color_space src_color = { .color = pl_color_space_bt709 };
-    struct pl_icc_color_space dst_color = { .color = pl_color_space_srgb };
-    struct pl_icc_result out;
-
-    if (pl_icc_update(sh, &src_color, &dst_color, &icc, &out, NULL)) {
-        pl_icc_apply(sh, &icc);
-        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
-            .shader = &sh,
-            .target = fbo,
-        }));
-    }
-
-    pl_dispatch_abort(dp, &sh);
-    pl_shader_obj_destroy(&icc);
-#endif
-
     // Test film grain synthesis
     pl_shader_obj grain = NULL;
     struct pl_film_grain_params grain_params = {
@@ -629,16 +607,18 @@ static void pl_shader_tests(pl_gpu gpu)
         }));
     }
 
-    grain_params.data.type = PL_FILM_GRAIN_H274;
-    grain_params.data.params.h274 = h274_grain_data;
-    grain_params.data.seed = rand();
+    if (gpu->glsl.compute) {
+        grain_params.data.type = PL_FILM_GRAIN_H274;
+        grain_params.data.params.h274 = h274_grain_data;
+        grain_params.data.seed = rand();
 
-    sh = pl_dispatch_begin(dp);
-    pl_shader_film_grain(sh, &grain, &grain_params);
-    REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
-        .shader = &sh,
-        .target = fbo,
-    }));
+        sh = pl_dispatch_begin(dp);
+        pl_shader_film_grain(sh, &grain, &grain_params);
+        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+            .shader = &sh,
+            .target = fbo,
+        }));
+    }
     pl_shader_obj_destroy(&grain);
 
     // Test custom shaders
@@ -1078,6 +1058,24 @@ static void pl_render_tests(pl_gpu gpu)
         pl_lut_free(&lut);
     }
 
+#ifdef PL_HAVE_LCMS
+    // Test ICC profiles
+    image.profile = TEST_PROFILE(sRGB_v2_nano_icc);
+    REQUIRE(pl_render_image(rr, &image, &target, &params));
+    image.profile = (struct pl_icc_profile) {0};
+
+    target.profile = TEST_PROFILE(sRGB_v2_nano_icc);
+    REQUIRE(pl_render_image(rr, &image, &target, &params));
+    target.profile = (struct pl_icc_profile) {0};
+
+    image.profile = TEST_PROFILE(sRGB_v2_nano_icc);
+    target.profile = image.profile;
+    REQUIRE(pl_render_image(rr, &image, &target, &params));
+    image.profile = (struct pl_icc_profile) {0};
+    target.profile = (struct pl_icc_profile) {0};
+
+#endif
+
     // Test overlays
     image.num_overlays = 1;
     image.overlays = &(struct pl_overlay) {
@@ -1132,6 +1130,17 @@ static void pl_render_tests(pl_gpu gpu)
         .frame_duration = 1.0 / 24.0,
     };
 
+    // Test large PTS jumps in frame mix
+    struct pl_frame_mix mix = (struct pl_frame_mix) {
+        .num_frames = 2,
+        .frames = (const struct pl_frame *[]) { &image, &image },
+        .signatures = (uint64_t[]) { 0xFFF1, 0xFFF2 },
+        .timestamps = (float[]) { -100, 100 },
+        .vsync_duration = 1.6,
+    };
+    REQUIRE(pl_render_image_mix(rr, &mix, &target, &mix_params));
+
+    // Test mixer queue
 #define NUM_MIX_FRAMES 20
     struct pl_source_frame srcframes[NUM_MIX_FRAMES+1];
     srcframes[NUM_MIX_FRAMES] = (struct pl_source_frame) {0};
@@ -1155,7 +1164,6 @@ static void pl_render_tests(pl_gpu gpu)
             pl_queue_push(queue, src); // push it anyway, for testing
     }
 
-    struct pl_frame_mix mix;
     while ((ret = pl_queue_update(queue, &mix, &qparams)) != PL_QUEUE_EOF) {
         if (ret == PL_QUEUE_MORE) {
             REQUIRE(qparams.pts > 0.0);
