@@ -42,6 +42,7 @@ void vk_dev_callback(struct vk_ctx *vk, vk_cb callback,
 struct vk_cmd {
     struct vk_cmdpool *pool; // pool it was allocated from
     VkQueue queue;           // the submission queue (for recording/pending)
+    int qindex;              // the index of `queue` in `pool`
     VkCommandBuffer buf;     // the command buffer itself
     VkFence fence;           // the fence guards cmd buffer reuse
     // The semaphores represent dependencies that need to complete before
@@ -122,7 +123,8 @@ void vk_cmdpool_destroy(struct vk_ctx *vk, struct vk_cmdpool *pool);
 
 // Fetch a command buffer from a command pool and begin recording to it.
 // Returns NULL on failure.
-struct vk_cmd *vk_cmd_begin(struct vk_ctx *vk, struct vk_cmdpool *pool);
+struct vk_cmd *vk_cmd_begin(struct vk_ctx *vk, struct vk_cmdpool *pool,
+                            pl_debug_tag debug_tag);
 
 // Finish recording a command buffer and submit it for execution. This function
 // takes over ownership of **cmd, and sets *cmd to NULL in doing so.
