@@ -424,8 +424,6 @@ static VkBool32 VKAPI_PTR vk_dbg_utils_cb(VkDebugUtilsMessageSeverityFlagBitsEXT
         break;
     }
 
-#endif
-
     enum pl_log_level lev;
     switch (sev) {
     case VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT:     lev = PL_LOG_ERR;   break;
@@ -437,7 +435,6 @@ static VkBool32 VKAPI_PTR vk_dbg_utils_cb(VkDebugUtilsMessageSeverityFlagBitsEXT
 
     pl_msg(log, lev, "vk %s", data->pMessage);
 
-#ifndef MSAN
     for (int i = 0; i < data->queueLabelCount; i++)
         pl_msg(log, lev, "    during %s", data->pQueueLabels[i].pLabelName);
     for (int i = 0; i < data->cmdBufLabelCount; i++)
@@ -449,7 +446,6 @@ static VkBool32 VKAPI_PTR vk_dbg_utils_cb(VkDebugUtilsMessageSeverityFlagBitsEXT
                obj->pObjectName ? obj->pObjectName : "anon",
                (unsigned long long) obj->objectHandle);
     }
-#endif
 
     // The return value of this function determines whether the call will
     // be explicitly aborted (to prevent GPU errors) or not. In this case,
@@ -458,10 +454,15 @@ static VkBool32 VKAPI_PTR vk_dbg_utils_cb(VkDebugUtilsMessageSeverityFlagBitsEXT
     bool is_error = (sev & VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT) &&
                     (msgType & VK_DEBUG_UTILS_MESSAGE_TYPE_VALIDATION_BIT_EXT);
 
-    if (is_error)
+    if (is_error) {
         pl_log_stack_trace(log, lev);
+        pl_debug_abort();
+        return true;
+    }
 
-    return is_error;
+#endif // MSAN
+
+    return false;
 }
 
 static PFN_vkGetInstanceProcAddr get_proc_addr_fallback(pl_log log,
@@ -665,6 +666,17 @@ debug_layers_done: ;
         }
     }
 
+#ifdef VK_KHR_portability_enumeration
+    // Required for macOS ( MoltenVK ) compatibility
+    for (int n = 0; n < num_exts_avail; n++) {
+        if (strcmp(VK_KHR_PORTABILITY_ENUMERATION_EXTENSION_NAME, exts_avail[n].extensionName) == 0) {
+            PL_ARRAY_APPEND(tmp, exts, VK_KHR_PORTABILITY_ENUMERATION_EXTENSION_NAME);
+            info.flags |= VK_INSTANCE_CREATE_ENUMERATE_PORTABILITY_BIT_KHR;
+            break;
+        }
+    }
+#endif
+
     // Add extra user extensions
     for (int i = 0; i < params->num_extensions; i++) {
         const char *ext = params->extensions[i];
@@ -1021,17 +1033,30 @@ static bool device_init(struct vk_ctx *vk, const struct pl_vulkan_params *params
                  (unsigned) qfs[i].queueFlags, (int) qfs[i].queueCount);
     }
 
-    int idx_gfx = -1, idx_comp = -1, idx_tf = -1;
-    idx_gfx = find_qf(qfs, qfnum, VK_QUEUE_GRAPHICS_BIT);
-    if (params->async_compute)
-        idx_comp = find_qf(qfs, qfnum, VK_QUEUE_COMPUTE_BIT);
-    if (params->async_transfer)
-        idx_tf = find_qf(qfs, qfnum, VK_QUEUE_TRANSFER_BIT);
+    VkQueueFlagBits gfx_flags = VK_QUEUE_GRAPHICS_BIT;
+    if (!params->async_compute)
+        gfx_flags |= VK_QUEUE_COMPUTE_BIT;
+
+    int idx_gfx  = find_qf(qfs, qfnum, gfx_flags);
+    int idx_comp = find_qf(qfs, qfnum, VK_QUEUE_COMPUTE_BIT);
+    int idx_tf   = find_qf(qfs, qfnum, VK_QUEUE_TRANSFER_BIT);
+    if (idx_tf < 0)
+        idx_tf = idx_comp;
+
+    if (!params->async_compute)
+        idx_comp = idx_gfx;
+    if (!params->async_transfer)
+        idx_tf = idx_gfx;
 
-    // Vulkan requires at least one GRAPHICS queue, so if this fails something
-    // is horribly wrong.
-    pl_assert(idx_gfx >= 0);
     PL_DEBUG(vk, "Using graphics queue %d", idx_gfx);
+    if (idx_tf != idx_gfx)
+        PL_INFO(vk, "Using async transfer (queue %d)", idx_tf);
+    if (idx_comp != idx_gfx)
+        PL_INFO(vk, "Using async compute (queue %d)", idx_comp);
+
+    // Vulkan requires at least one GRAPHICS+COMPUTE queue, so if this fails
+    // something is horribly wrong.
+    pl_assert(idx_gfx >= 0 && idx_comp >= 0 && idx_tf >= 0);
 
     // If needed, ensure we can actually present to the surface using this queue
     if (params->surface) {
@@ -1044,16 +1069,6 @@ static bool device_init(struct vk_ctx *vk, const struct pl_vulkan_params *params
         }
     }
 
-    // Fall back to supporting compute shaders via the graphics pool for
-    // devices which support compute shaders but not async compute.
-    if (idx_comp < 0 && qfs[idx_gfx].queueFlags & VK_QUEUE_COMPUTE_BIT)
-        idx_comp = idx_gfx;
-
-    if (idx_tf >= 0 && idx_tf != idx_gfx)
-        PL_INFO(vk, "Using async transfer (queue %d)", idx_tf);
-    if (idx_comp >= 0 && idx_comp != idx_gfx)
-        PL_INFO(vk, "Using async compute (queue %d)", idx_comp);
-
     // Now that we know which QFs we want, we can create the logical device
     qinfo_arr_t qinfos = {0};
     add_qinfo(tmp, &qinfos, qfs, idx_gfx, params->queue_count);
@@ -1073,6 +1088,8 @@ static bool device_init(struct vk_ctx *vk, const struct pl_vulkan_params *params
     // Add all extensions we need
     if (params->surface)
         PL_ARRAY_APPEND(vk->alloc, vk->exts, VK_KHR_SWAPCHAIN_EXTENSION_NAME);
+    if (vk->api_ver < VK_API_VERSION_1_2)
+        PL_ARRAY_APPEND(vk->alloc, vk->exts, VK_KHR_TIMELINE_SEMAPHORE_EXTENSION_NAME);
 
     // Keep track of all optional function pointers associated with extensions
     PL_ARRAY(const struct vk_fun *) ext_funs = {0};
@@ -1225,6 +1242,108 @@ error:
     return false;
 }
 
+static void lock_queue_internal(void *priv, int qf, int qidx)
+{
+    struct vk_ctx *vk = priv;
+    pl_mutex_lock(&vk->queue_locks.elem[qf][qidx]);
+}
+
+static void unlock_queue_internal(void *priv, int qf, int qidx)
+{
+    struct vk_ctx *vk = priv;
+    pl_mutex_unlock(&vk->queue_locks.elem[qf][qidx]);
+}
+
+static void lock_queue(pl_vulkan pl_vk, int qf, int qidx)
+{
+    struct vk_ctx *vk = PL_PRIV(pl_vk);
+    vk->lock_queue(vk->queue_ctx, qf, qidx);
+}
+
+static void unlock_queue(pl_vulkan pl_vk, int qf, int qidx)
+{
+    struct vk_ctx *vk = PL_PRIV(pl_vk);
+    vk->unlock_queue(vk->queue_ctx, qf, qidx);
+}
+
+static bool finalize_context(struct pl_vulkan *pl_vk, int max_glsl_version)
+{
+    struct vk_ctx *vk = PL_PRIV(pl_vk);
+
+    pl_assert(vk->pool_graphics);
+    pl_assert(vk->pool_compute);
+    pl_assert(vk->pool_transfer);
+
+    vk->ma = vk_malloc_create(vk);
+    if (!vk->ma)
+        return false;
+
+    pl_vk->gpu = pl_gpu_create_vk(vk);
+    if (!pl_vk->gpu)
+        return false;
+
+    // Blacklist / restrict features
+    if (max_glsl_version) {
+        struct pl_glsl_version *glsl = (struct pl_glsl_version *) &pl_vk->gpu->glsl;
+        glsl->version = PL_MIN(glsl->version, max_glsl_version);
+        glsl->version = PL_MAX(glsl->version, 140); // required for GL_KHR_vulkan_glsl
+        PL_INFO(vk, "Restricting GLSL version to %d... new version is %d",
+                max_glsl_version, glsl->version);
+    }
+
+    // Expose the resulting vulkan objects
+    pl_vk->instance = vk->inst;
+    pl_vk->phys_device = vk->physd;
+    pl_vk->device = vk->dev;
+    pl_vk->api_version = vk->api_ver;
+    pl_vk->extensions = vk->exts.elem;
+    pl_vk->num_extensions = vk->exts.num;
+    pl_vk->features = &vk->features;
+    pl_vk->num_queues = vk->pools.num;
+    pl_vk->queues = pl_calloc_ptr(vk->alloc, vk->pools.num, pl_vk->queues);
+    pl_vk->lock_queue = lock_queue;
+    pl_vk->unlock_queue = unlock_queue;
+
+    uint32_t max_qf = 0;
+    for (int i = 0; i < vk->pools.num; i++) {
+        struct pl_vulkan_queue *queues = (struct pl_vulkan_queue *) pl_vk->queues;
+        queues[i] = (struct pl_vulkan_queue) {
+            .index = vk->pools.elem[i]->qf,
+            .count = vk->pools.elem[i]->num_queues,
+        };
+
+        if (vk->pools.elem[i] == vk->pool_graphics)
+            pl_vk->queue_graphics = queues[i];
+        if (vk->pools.elem[i] == vk->pool_compute)
+            pl_vk->queue_compute = queues[i];
+        if (vk->pools.elem[i] == vk->pool_transfer)
+            pl_vk->queue_transfer = queues[i];
+
+        max_qf = PL_MAX(max_qf, queues[i].index);
+    }
+
+    if (!vk->lock_queue) {
+        vk->queue_locks.num = max_qf + 1;
+        PL_ARRAY_RESIZE(vk->alloc, vk->queue_locks, vk->queue_locks.num);
+
+        for (int i = 0; i < vk->pools.num; i++) {
+            struct vk_cmdpool *pool = vk->pools.elem[i];
+            pl_mutex **locks = vk->queue_locks.elem;
+            locks[pool->qf] = pl_alloc(vk->alloc, pool->num_queues * sizeof(pl_mutex));
+            for (int n = 0; n < pool->num_queues; n++)
+                pl_mutex_init(&locks[pool->qf][n]);
+        }
+
+        vk->lock_queue = lock_queue_internal;
+        vk->unlock_queue = unlock_queue_internal;
+        vk->queue_ctx = vk;
+    }
+
+    pl_assert(vk->lock_queue);
+    pl_assert(vk->unlock_queue);
+    return true;
+}
+
 pl_vulkan pl_vulkan_create(pl_log log, const struct pl_vulkan_params *params)
 {
     params = PL_DEF(params, &pl_vulkan_default_params);
@@ -1300,7 +1419,7 @@ pl_vulkan pl_vulkan_create(pl_log log, const struct pl_vulkan_params *params)
     PL_INFO(vk, "    Device ID: %x:%x", (unsigned) prop.properties.vendorID,
             (unsigned) prop.properties.deviceID);
     PL_INFO(vk, "    Device UUID: %s", PRINT_UUID(id_props.deviceUUID));
-    PL_INFO(vk, "    Driver version: %d", (int) prop.properties.driverVersion);
+    PL_INFO(vk, "    Driver version: %x", (unsigned) prop.properties.driverVersion);
     PL_INFO(vk, "    API version: %d.%d.%d", PRINTF_VER(prop.properties.apiVersion));
 
     // Needed by device_init
@@ -1322,48 +1441,9 @@ pl_vulkan pl_vulkan_create(pl_log log, const struct pl_vulkan_params *params)
     if (!device_init(vk, params))
         goto error;
 
-    vk->ma = vk_malloc_create(vk);
-    if (!vk->ma)
+    if (!finalize_context(pl_vk, params->max_glsl_version))
         goto error;
 
-    pl_vk->gpu = pl_gpu_create_vk(vk);
-    if (!pl_vk->gpu)
-        goto error;
-
-    // Blacklist / restrict features
-    if (params->max_glsl_version) {
-        struct pl_glsl_version *glsl = (struct pl_glsl_version *) &pl_vk->gpu->glsl;
-        glsl->version = PL_MIN(glsl->version, params->max_glsl_version);
-        glsl->version = PL_MAX(glsl->version, 140); // required for GL_KHR_vulkan_glsl
-        PL_INFO(vk, "Restricting GLSL version to %d... new version is %d",
-                params->max_glsl_version, glsl->version);
-    }
-
-    // Expose the resulting vulkan objects
-    pl_vk->instance = vk->inst;
-    pl_vk->phys_device = vk->physd;
-    pl_vk->device = vk->dev;
-    pl_vk->api_version = vk->api_ver;
-    pl_vk->extensions = vk->exts.elem;
-    pl_vk->num_extensions = vk->exts.num;
-    pl_vk->features = &vk->features;
-    pl_vk->num_queues = vk->pools.num;
-    pl_vk->queues = pl_calloc_ptr(pl_vk, vk->pools.num, pl_vk->queues);
-    for (int i = 0; i < vk->pools.num; i++) {
-        struct pl_vulkan_queue *queues = (struct pl_vulkan_queue *) pl_vk->queues;
-        queues[i] = (struct pl_vulkan_queue) {
-            .index = vk->pools.elem[i]->qf,
-            .count = vk->pools.elem[i]->num_queues,
-        };
-
-        if (vk->pools.elem[i] == vk->pool_graphics)
-            pl_vk->queue_graphics = queues[i];
-        if (vk->pools.elem[i] == vk->pool_compute && vk->pool_compute != vk->pool_graphics)
-            pl_vk->queue_compute = queues[i];
-        if (vk->pools.elem[i] == vk->pool_compute)
-            pl_vk->queue_compute = queues[i];
-    }
-
     return pl_vk;
 
 error:
@@ -1387,6 +1467,9 @@ pl_vulkan pl_vulkan_import(pl_log log, const struct pl_vulkan_import_params *par
         .physd = params->phys_device,
         .dev = params->device,
         .GetInstanceProcAddr = get_proc_addr_fallback(log, params->get_proc_addr),
+        .lock_queue = params->lock_queue,
+        .unlock_queue = params->unlock_queue,
+        .queue_ctx = params->queue_ctx,
     };
 
     pl_mutex_init_type(&vk->lock, PL_MUTEX_RECURSIVE);
@@ -1495,16 +1578,22 @@ pl_vulkan pl_vulkan_import(pl_log log, const struct pl_vulkan_import_params *par
     struct {
         const struct pl_vulkan_queue *info;
         struct vk_cmdpool **pool;
+        VkQueueFlagBits flags; // *any* of these flags provide the cap
     } qinfos[] = {
         {
             .info = &params->queue_graphics,
             .pool = &vk->pool_graphics,
+            .flags = VK_QUEUE_GRAPHICS_BIT,
         }, {
             .info = &params->queue_compute,
             .pool = &vk->pool_compute,
+            .flags = VK_QUEUE_COMPUTE_BIT,
         }, {
             .info = &params->queue_transfer,
             .pool = &vk->pool_transfer,
+            .flags = VK_QUEUE_TRANSFER_BIT |
+                     VK_QUEUE_GRAPHICS_BIT |
+                     VK_QUEUE_COMPUTE_BIT,
         }
     };
 
@@ -1514,6 +1603,9 @@ pl_vulkan pl_vulkan_import(pl_log log, const struct pl_vulkan_import_params *par
         if (!qinfos[i].info->count)
             continue;
 
+        // API sanity check
+        pl_assert(qfs[qf].queueFlags & qinfos[i].flags);
+
         // See if we already created a pool for this queue family
         for (int j = 0; j < i; j++) {
             if (qinfos[j].info->count && qinfos[j].info->index == qf) {
@@ -1532,60 +1624,29 @@ pl_vulkan pl_vulkan_import(pl_log log, const struct pl_vulkan_import_params *par
             goto error;
         PL_ARRAY_APPEND(vk->alloc, vk->pools, *pool);
 
+        // Pre-emptively set "lower priority" pools as well
+        for (int j = i+1; j < PL_ARRAY_SIZE(qinfos); j++) {
+            if (qfs[qf].queueFlags & qinfos[j].flags)
+                *qinfos[j].pool = *pool;
+        }
+
 next_qf: ;
     }
 
-    if (!vk->pool_compute && (vk->pool_graphics->props.queueFlags & VK_QUEUE_COMPUTE_BIT))
-        vk->pool_compute = vk->pool_graphics;
-
-    vk->ma = vk_malloc_create(vk);
-    if (!vk->ma)
+    if (!vk->pool_graphics) {
+        PL_ERR(vk, "No valid queues provided?");
         goto error;
-
-    pl_vk->gpu = pl_gpu_create_vk(vk);
-    if (!pl_vk->gpu)
-        goto error;
-
-    // Blacklist / restrict features
-    if (params->max_glsl_version) {
-        struct pl_glsl_version *glsl = (struct pl_glsl_version *) &pl_vk->gpu->glsl;
-        glsl->version = PL_MIN(glsl->version, params->max_glsl_version);
-        PL_INFO(vk, "Restricting GLSL version to %d... new version is %d",
-                params->max_glsl_version, glsl->version);
     }
 
-    // Expose the resulting vulkan objects
-    pl_vk->instance = vk->inst;
-    pl_vk->phys_device = vk->physd;
-    pl_vk->device = vk->dev;
-    pl_vk->api_version = vk->api_ver;
-    pl_vk->extensions = vk->exts.elem;
-    pl_vk->num_extensions = vk->exts.num;
-    pl_vk->features = &vk->features;
-    pl_vk->num_queues = vk->pools.num;
-    pl_vk->queues = pl_calloc_ptr(pl_vk, vk->pools.num, pl_vk->queues);
-    struct pl_vulkan_queue *queues = (struct pl_vulkan_queue *) pl_vk->queues;
-
-    for (int i = 0; i < vk->pools.num; i++) {
-        queues[i] = (struct pl_vulkan_queue) {
-            .index = vk->pools.elem[i]->qf,
-            .count = vk->pools.elem[i]->num_queues,
-        };
-
-        if (vk->pools.elem[i] == vk->pool_graphics)
-            pl_vk->queue_graphics = queues[i];
-        if (vk->pools.elem[i] == vk->pool_compute && vk->pool_compute != vk->pool_graphics)
-            pl_vk->queue_compute = queues[i];
-        if (vk->pools.elem[i] == vk->pool_compute)
-            pl_vk->queue_compute = queues[i];
-    }
+    if (!finalize_context(pl_vk, params->max_glsl_version))
+        goto error;
 
     pl_free(tmp);
     return pl_vk;
 
 error:
     PL_FATAL(vk, "Failed importing vulkan device");
-    pl_free(tmp);
     pl_vulkan_destroy((pl_vulkan *) &pl_vk);
+    pl_free(tmp);
     return NULL;
 }
