@@ -169,23 +169,13 @@ struct vk_cmd *_begin_cmd(pl_gpu gpu, enum queue_type type, const char *label,
     case ANY:      pool = p->cmd ? p->cmd->pool : vk->pool_graphics; break;
     case GRAPHICS: pool = vk->pool_graphics; break;
     case COMPUTE:  pool = vk->pool_compute;  break;
-
-    // GRAPHICS and COMPUTE also imply TRANSFER capability (vulkan spec)
-    case TRANSFER:
-        pool = vk->pool_transfer;
-        if (!pool)
-            pool = vk->pool_compute;
-        if (!pool)
-            pool = vk->pool_graphics;
-        break;
-
-    default:
-        pl_unreachable();
+    case TRANSFER: pool = vk->pool_transfer; break;
+    default: pl_unreachable();
     }
 
     if (!p->cmd || p->cmd->pool != pool) {
         vk_cmd_submit(vk, &p->cmd);
-        p->cmd = vk_cmd_begin(vk, pool);
+        p->cmd = vk_cmd_begin(vk, pool, label);
         if (!p->cmd) {
             pl_mutex_unlock(&p->recording);
             return NULL;
@@ -317,9 +307,14 @@ static pl_handle_caps vk_tex_handle_caps(struct vk_ctx *vk, bool import)
 {
     pl_handle_caps caps = 0;
 
-    bool has_drm_mods = vk->GetImageDrmFormatModifierPropertiesEXT;
     for (int i = 0; vk_mem_handle_list[i]; i++) {
         enum pl_handle_type handle_type = vk_mem_handle_list[i];
+        if (handle_type == PL_HANDLE_DMA_BUF && !vk->GetImageDrmFormatModifierPropertiesEXT) {
+            PL_DEBUG(vk, "Tex caps for %s (0x%x) unsupported: no DRM modifiers",
+                     vk_handle_name(vk_mem_handle_type(PL_HANDLE_DMA_BUF)),
+                     (unsigned int) PL_HANDLE_DMA_BUF);
+            continue;
+        }
 
         // Query whether creation of a "basic" dummy texture would work
         VkPhysicalDeviceImageDrmFormatModifierInfoEXT drm_pinfo = {
@@ -342,7 +337,7 @@ static pl_handle_caps vk_tex_handle_caps(struct vk_ctx *vk, bool import)
             .usage = VK_IMAGE_USAGE_TRANSFER_DST_BIT,
         };
 
-        if (handle_type == PL_HANDLE_DMA_BUF && has_drm_mods) {
+        if (handle_type == PL_HANDLE_DMA_BUF) {
             vk_link_struct(&pinfo, &drm_pinfo);
             pinfo.tiling = VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT;
         }
@@ -358,11 +353,6 @@ static pl_handle_caps vk_tex_handle_caps(struct vk_ctx *vk, bool import)
 
         VkResult res;
         res = vk->GetPhysicalDeviceImageFormatProperties2KHR(vk->physd, &pinfo, &props);
-        if (res != VK_SUCCESS && handle_type == PL_HANDLE_DMA_BUF && !has_drm_mods) {
-            // Try again with VK_IMAGE_TILING_LINEAR, as a dumb hack
-            pinfo.tiling = VK_IMAGE_TILING_LINEAR;
-            res = vk->GetPhysicalDeviceImageFormatProperties2KHR(vk->physd, &pinfo, &props);
-        }
         if (res != VK_SUCCESS) {
             PL_DEBUG(vk, "Tex caps for %s (0x%x) unsupported: %s",
                      vk_handle_name(ext_pinfo.handleType),
@@ -450,16 +440,16 @@ pl_gpu pl_gpu_create_vk(struct vk_ctx *vk)
     gpu->glsl = (struct pl_glsl_version) {
         .version = 450,
         .vulkan = true,
+        .compute = true,
+        .max_shmem_size = vk->limits.maxComputeSharedMemorySize,
+        .max_group_threads = vk->limits.maxComputeWorkGroupInvocations,
+        .max_group_size = {
+            vk->limits.maxComputeWorkGroupSize[0],
+            vk->limits.maxComputeWorkGroupSize[1],
+            vk->limits.maxComputeWorkGroupSize[2],
+        },
     };
 
-    if (vk->pool_compute) {
-        gpu->glsl.compute = true;
-        gpu->glsl.max_shmem_size = vk->limits.maxComputeSharedMemorySize;
-        gpu->glsl.max_group_threads = vk->limits.maxComputeWorkGroupInvocations;
-        for (int i = 0; i < 3; i++)
-            gpu->glsl.max_group_size[i] = vk->limits.maxComputeWorkGroupSize[i];
-    }
-
     VkShaderStageFlags req_stages = VK_SHADER_STAGE_FRAGMENT_BIT |
                                     VK_SHADER_STAGE_COMPUTE_BIT;
     VkSubgroupFeatureFlags req_flags = VK_SUBGROUP_FEATURE_BASIC_BIT |
@@ -514,7 +504,7 @@ pl_gpu pl_gpu_create_vk(struct vk_ctx *vk)
             vk->limits.maxComputeWorkGroupCount[2],
         },
         .fragment_queues    = vk->pool_graphics->num_queues,
-        .compute_queues     = vk->pool_compute ? vk->pool_compute->num_queues : 0,
+        .compute_queues     = vk->pool_compute->num_queues,
     };
 
     gpu->export_caps.buf = vk_malloc_handle_caps(vk->ma, false);
@@ -751,7 +741,7 @@ struct vk_cmd *pl_vk_steal_cmd(pl_gpu gpu)
     struct vk_cmdpool *pool = vk->pool_graphics;
     if (!cmd || cmd->pool != pool) {
         vk_cmd_submit(vk, &cmd);
-        cmd = vk_cmd_begin(vk, pool);
+        cmd = vk_cmd_begin(vk, pool, NULL);
     }
 
     return cmd;
