@@ -31,12 +31,6 @@ void vk_tex_barrier(pl_gpu gpu, struct vk_cmd *cmd, pl_tex tex,
         vk_cmd_dep(cmd, stage, tex_vk->ext_deps.elem[i]);
     tex_vk->ext_deps.num = 0;
 
-    // Transitioning to VK_IMAGE_LAYOUT_UNDEFINED is a pseudo-operation for
-    // render passes, which means that we don't actually need to perform
-    // any image layout transition
-    if (layout == VK_IMAGE_LAYOUT_UNDEFINED)
-        layout = tex_vk->layout;
-
     struct vk_sync_scope last;
     bool is_trans = layout != tex_vk->layout;
     last = vk_sem_barrier(vk, cmd, &tex_vk->sem, stage, access, is_trans || export);
@@ -357,13 +351,6 @@ pl_tex vk_tex_create(pl_gpu gpu, const struct pl_tex_params *params)
         .pQueueFamilyIndices = qfs,
     };
 
-    bool has_drm_mods = vk->GetImageDrmFormatModifierPropertiesEXT;
-    if (handle_type == PL_HANDLE_DMA_BUF && !has_drm_mods && !p->warned_modless) {
-        PL_WARN(gpu, "Using legacy hacks for DMA buffers without modifiers. "
-                "May result in corruption!");
-        p->warned_modless = true;
-    }
-
     struct vk_malloc_params mparams = {
         .optimal = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
         .export_handle = params->export_handle,
@@ -372,27 +359,12 @@ pl_tex vk_tex_create(pl_gpu gpu, const struct pl_tex_params *params)
     };
 
     if (params->import_handle == PL_HANDLE_DMA_BUF) {
-        if (has_drm_mods) {
-
-            // We have VK_EXT_image_drm_format_modifier, so we can use
-            // format modifiers properly
-            vk_link_struct(&iinfo, &drm_explicit);
-            iinfo.tiling = VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT;
-            mparams.shared_mem.offset = 0x0; // handled via plane offsets
-
-        } else {
-
-            // Legacy fallback for older drivers. Based on hacks and guesswork.
-            switch (drm_explicit.drmFormatModifier) {
-            case DRM_FORMAT_MOD_LINEAR:
-                iinfo.tiling = VK_IMAGE_TILING_LINEAR;
-                break;
-            }
-
-        }
+        vk_link_struct(&iinfo, &drm_explicit);
+        iinfo.tiling = VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT;
+        mparams.shared_mem.offset = 0x0; // handled via plane offsets
     }
 
-    if (params->export_handle == PL_HANDLE_DMA_BUF && has_drm_mods) {
+    if (params->export_handle == PL_HANDLE_DMA_BUF) {
         vk_link_struct(&iinfo, &drm_list);
         iinfo.tiling = VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT;
     }
@@ -411,7 +383,7 @@ pl_tex vk_tex_create(pl_gpu gpu, const struct pl_tex_params *params)
         .handleType = ext_info.handleTypes,
     };
 
-    if (handle_type == PL_HANDLE_DMA_BUF && has_drm_mods)
+    if (handle_type == PL_HANDLE_DMA_BUF)
         vk_link_struct(&ext_pinfo, &drm_pinfo);
 
     VkPhysicalDeviceImageFormatInfo2KHR pinfo = {
@@ -684,7 +656,7 @@ static enum queue_type vk_img_copy_queue(pl_gpu gpu, pl_tex tex,
 
     const struct pl_tex_vk *tex_vk = PL_PRIV(tex);
     enum queue_type queue = tex_vk->transfer_queue;
-    if (queue != TRANSFER || !vk->pool_transfer)
+    if (queue != TRANSFER)
         return queue;
 
     VkExtent3D alignment = vk->pool_transfer->props.minImageTransferGranularity;
